package net.pricefx.lmformulaengine.model;

import com.googlecode.genericdao.search.Filter;
import org.jetbrains.annotations.Nullable;

import java.util.*;

import static net.pricefx.common.api.PriceFxInterface.*;


public class MapExtended extends LinkedHashMap<String, Object> {
    private String typeCode;
    private Object instanceIdentifier;

    public MapExtended(Map<String, Object> value, String typeCode, Object instanceIdentifier) {
        super(value);
        this.typeCode = typeCode;
        this.instanceIdentifier = instanceIdentifier;
    }

    private void checkFilters(List<Filter> filters) {
        if (filters == null) {
            return;
        }

        for (Filter filter : filters) {
            if (FIELD_NAME.equals(filter.getProperty()) && filter.getOperator() == Filter.OP_EQUAL) {
                instanceIdentifier = filter.getValue();
                return;
            } else if (filter.getOperator() == Filter.OP_AND && filter.getValue() instanceof List) {
                checkFilters((List<Filter>) filter.getValue());
            }
        }
    }

    public MapExtended(Map<String, Object> value, String typeCode, Filter... filters) {
        this(value, typeCode, (String) null);
        if (filters != null) {
            checkFilters(Arrays.asList(filters));
        }
    }

    public static List<MapExtended> convertList(List<Map<String, Object>> values, String typeCode, Object uniqueName) {
        List<MapExtended> result = new ArrayList<>();
        for (Map<String, Object> value : values) {
            result.add(new MapExtended(value, typeCode, uniqueName));
        }
        return result;
    }


    public static List<MapExtended> convertList(List<Map<String, Object>> values, String typeCode,  Filter... filters) {
        List<MapExtended> result = new ArrayList<>();
        for (Map<String, Object> value : values) {
            result.add(new MapExtended(value, typeCode, filters));
        }
        return result;
    }



    public String getTypeCode() {
        return typeCode;
    }

    @Nullable
    public Object getInstanceIdentifier() {
        return instanceIdentifier;
    }
}
