package net.pricefx.logic.core.model;

import com.fasterxml.jackson.databind.node.ObjectNode;
import net.pricefx.pckg.filesystem.ItemMarkers;
import net.pricefx.pckg.processing.ProcessingMarkers;
import net.pricefx.pckg.transform.TransformCalculationLogic;
import net.pricefx.pckg.transform.TransformWorkflowFormula;
import net.pricefx.pckg.transform.descriptor.TypeDescriptor;
import net.pricefx.restapiclient.model.BaseMap;

import java.math.BigDecimal;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static net.pricefx.restapiclient.PfxClient.DEFAULT_OBJECT_MAPPER;

public class ObjectHolder {

	protected Map<String, Object> holder;
	protected Path dirPath;


	public ObjectHolder() {
		this(new HashMap<>());
	}

	public ObjectHolder(Map<String, Object> holder) {
		this.holder = holder;
	}

	public ObjectHolder(ObjectNode objectNode) {
		holder = DEFAULT_OBJECT_MAPPER.convertValue(objectNode, BaseMap.class);
	}

	public ObjectHolder(ObjectNode objectNode, Path modulePath) {
		this(objectNode);
		setDirPathBasedOnFsId(modulePath);
	}

	public static ObjectHolder[] convertObjectHolderArray(TypeDescriptor typeDescriptor, List<ObjectNode> nodes, Path modulePath) {
		ObjectHolder[] objects;
		if (typeDescriptor == TransformCalculationLogic.DESCRIPTOR || typeDescriptor == TransformWorkflowFormula.DESCRIPTOR) {
			objects = new AbstractFormula[nodes.size()];
			for (int i=0; i<nodes.size(); i++) {
				objects[i] = AbstractFormula.createLogic(nodes.get(i), typeDescriptor, modulePath);
			}
		} else {
			objects = new ObjectHolder[nodes.size()];
			for (int i=0; i<nodes.size(); i++) {
				objects[i] = new ObjectHolder(nodes.get(i), modulePath);
			}
		}
		return objects;
	}

	public void setDirPath(Path dirPath) {
		this.dirPath = dirPath;
	}

	public Path getDirPath() {
		return dirPath;
	}

	public String getDirPathBasedOnFsId() {
		return (String) ((Map) holder.get(ProcessingMarkers.FIELD_PROCESS)).get(ItemMarkers.FIELD_fileSystemId);
	}

	public void setDirPathBasedOnFsId(Path modulePath) {
		String value = getDirPathBasedOnFsId();
		if (value != null) {
			setDirPath(modulePath.resolve(value instanceof String ? (String) value : null));
		}
	}

	public ObjectNode getNode() {
		return DEFAULT_OBJECT_MAPPER.convertValue(holder, ObjectNode.class);
	}

	public String getTextValue(String property) {
		Object value = holder.get(property);
		return value instanceof String ? (String) value : null;
	}

	public Long getLongValue(String property) {
		Object value = holder.get(property);
		if (value instanceof Long) {
			return (Long) value;
		} else if (value instanceof Integer) {
			return ((Integer) value).longValue();
		} else if (value instanceof BigDecimal) {
			return ((BigDecimal) value).longValue();
		}
		return null;
	}

	public BigDecimal getDecimalValue(String property) {
		Object value = holder.get(property);
		if (value instanceof Long) {
			return new BigDecimal((Long) value);
		} else if (value instanceof Integer) {
			return new BigDecimal((Integer) value);
		} else if (value instanceof BigDecimal) {
			return (BigDecimal) value;
		}
		return null;
	}

	public Boolean getBooleanValue(String property) {
		Object value = holder.get(property);
		return value instanceof Boolean ? (Boolean) value : null;
	}

	public Map<String, Object> getObjectValue(String property) {
		Object value = holder.get(property);
		return value instanceof Map ? (Map<String, Object>) value : null;
	}

	public <T> List<T> getArrayValue(String property) {
		Object value = holder.get(property);
		return value instanceof List ? (List<T>) value : null;
	}



	protected void setTextValue(String property, String value) {
		if (value == null) {
			holder.remove(property);
		} else {
			holder.put(property, value);
		}
	}

	protected void setLongValue(String property, Long value) {
		if (value == null) {
			holder.remove(property);
		} else {
			holder.put(property, value);
		}
	}

	protected void setDecimalValue(String property, BigDecimal value) {
		if (value == null) {
			holder.remove(property);
		} else {
			holder.put(property, value);
		}
	}

	protected void setBooleanValue(String property, Boolean value) {
		if (value == null) {
            holder.remove(property);
		} else {
			holder.put(property, value);
		}
	}

	protected void setObjectValue(String property, Map<String, Object> value) {
		if (value == null) {
			holder.remove(property);
		} else {
			holder.put(property, value);
		}
	}

	protected void setArrayValue(String property, List<String> value) {
		if (value == null) {
			holder.remove(property);
		} else {
			holder.put(property, value);
		}
	}

	public Map<String, Object> getHolder() {
		return holder;
	}
}
